#!/usr/bin/env ruby
# $Id: riff.rb,v 1.11 2006/04/30 08:55:04 hiroaki Exp $
#
# Riff - Now Biff yelps with RSS

require 'net/pop'
require 'kconv'
require 'webrick'
require 'uri'
require 'yaml'
require 'optparse'

include WEBrick

$riff_version='$Revision: 1.11 $'
$riff_desc='$Id: riff.rb,v 1.11 2006/04/30 08:55:04 hiroaki Exp $'

conf_file='~/.riff'
port=11080

# Analyze command line option
Version=$riff_version
if ARGV.length > 0
  ARGV.options {|opt|
    opt.on('-f Filename', 'Specify the configuration file.') {|v| conf_file=v}
    opt.on('-p PortNumber', 'Specify the port number riff uses.') {|v| port=v}
    
    opt.parse!
  }
end

# Configuration file
# Use YAML
#
# ex)
# entry name:
#  type: pop
#  server: mail.example.com
#  port: 110
#  user: foo
#  passwd: bar

account_list=''
File.open(File.expand_path(conf_file)){|fo|
  account_list=YAML.load(fo.read())
}

#
#
#

def xmlencode(str)
  return str.gsub(/&/, "&amp;").gsub(/</,"&lt;").gsub(/>/, "&gt;").gsub(/"/, "&quot;").gsub( /'/, "&apos;")
end

#
#
#

def print_header(account_data)
  title="#{account_data["user"]}\'s status on #{account_data["server"]}"
  link="pop://#{account_data["user"]}@#{account_data["server"]}"
  if account_data["port"] != "110" then
    link += ":#{account_data["port"]}"
  end
  description=title
  ret = '<?xml version="1.0" encoding="UTF-8"?>'
  ret += "\n"
  ret += '<rss version="2.0">'
  ret += "\n<channel>\n"
  ret += "<title>#{xmlencode(title)}</title>\n"
  ret += "<link>#{URI.escape(link)}</link>\n"
  ret += "<description>#{xmlencode(description)}</description>\n"
  ret += "<lastBuildDate>"
  ret += Time.now.strftime("%a, %d %b %Y %T %z")
  ret += "</lastBuildDate>\n"
  ret += "<generator>#{$riff_desc}</generator>"
  ret += "\n"
  return ret
end

def print_footer
  return "</channel>\n</rss>\n"
end

def print_item(items)
  ret = "<item>\n"
  ret += "<title>#{xmlencode(items["title"]).toutf8}</title>\n" if items["title"] != ""
  ret += "<author>#{xmlencode(items["author"])}</author>\n" if items["author"] != ""
  ret += "<description>#{xmlencode(items["description"]).toutf8}</description>\n" if items["description"] != ""
  ret += "<pubDate>#{items["pubDate"]}</pubDate>\n" if items["pubDate"] != ""
  if items["guid"] != "" then
    ret += '<guid isPermaLink="false">'
    ret += items["guid"]
    ret += "</guid>\n"
  end
  ret += "</item>\n"
  return ret
end


def print_pop(account_data)
  ret = ""
  Net::POP3.start( account_data["server"], account_data["port"], 
                   account_data["user"], account_data["passwd"]) { |pop3|
    pop3.each_mail do |m| 
      l=m.header.gsub(/\r\n\s+/, ' ').split(/\r\n/)
      subject=from=date=desc=""
      l.each {|ll|
        case ll
        when /^Subject:\s*/
          subject=$'
        when /^From:\s*/
          desc=$'
          if (desc=~/(.+?)\s*<(\S+@\S+)>/)
            from=$2+" ("+$1+")"
          else
            from=desc
          end
        when /^Date:\s*/
          date=$'.gsub(/\s+\(\S+\)/,"")
        end
      }
      ret += print_item({"title"=>subject, "author"=>from, 
                          "description"=>desc, "pubDate"=>date,
                          "guid"=>m.unique_id})
    end
  }
  return ret
end


#
# GET /localhost/riff/foo
# foo points the entry of foo
#
riff_proc=Proc.new {|req, resp|
  req.path_info =~ /^\//
  account_key = $'
  resp.body= print_header(account_list[account_key]) +
  print_pop(account_list[account_key]) +
  print_footer
}
server=HTTPServer.new( :Port => port, :BindAddress => 'localhost')
['INT','TERM'].each { |signal|
  trap(signal){ server.shutdown}
}
server.mount("/riff", HTTPServlet::ProcHandler.new(riff_proc))
server.start
